@@ -586,7 +586,9 @@ class DataPlate:
                               taken as a local file to upload (default: None).
             parameters (dict): The dictionary of parameters to pass to the notebook (default: {}).
             instance_type (str): The SageMaker instance to use for executing the job (default: ml.m5.large).
-
+            max_time_limit_minutes : maximum minutes to run before force stop
+            securityGroupIds : a list of securityGroup Ids of aws for the processing job to communicate with, in case communication with other resources, e.g. internal dataplate service, is needed
+            subnets : a list of subnets of aws for the processing job to communicate with
         Returns:
             The name of the processing job created to run the notebook.
         """
@@ -700,12 +702,15 @@ class DataPlate:
                 if r.text:
                     rJson = json.loads(r.text)
                     logging.info('list notebook runs finished successfully.')
-                    df = pd.read_json(rJson)
-                    df['Created'] = pd.to_datetime(df['Created'], unit='ms',utc=True)
-                    df['Start'] = pd.to_datetime(df['Start'], unit='ms', utc=True)
-                    df['End'] = pd.to_datetime(df['End'], unit='ms', utc=True)
-                    df['Elapsed'] = pd.to_timedelta(df['Elapsed'], unit='ms')
-                    return df
+                    try:
+                        df = pd.read_json(rJson)
+                        df['Created'] = pd.to_datetime(df['Created'], unit='ms',utc=True)
+                        df['Start'] = pd.to_datetime(df['Start'], unit='ms', utc=True)
+                        df['End'] = pd.to_datetime(df['End'], unit='ms', utc=True)
+                        df['Elapsed'] = pd.to_timedelta(df['Elapsed'], unit='ms')
+                        return df
+                    except Exception as e:
+                        logging.info('No notebook runs were found.')
                 else:
                     logging.exception('Could not find proper output, please check your parameters')
                 # return r.text
@@ -898,8 +903,11 @@ class DataPlate:
                 if r.text:
                     rJson = json.loads(r.text)
                     logging.info('list datasets finished')
-                    df = pd.read_json(json.dumps(rJson))
-                    return df
+                    try:
+                        df = pd.read_json(json.dumps(rJson))
+                        return df
+                    except Exception as e:
+                        logging.info('No datasets were found.')
                 else:
                     logging.exception('Could not find proper output, please check your parameters')
                 # return r.text
@@ -911,4 +919,324 @@ class DataPlate:
 
         except Exception as e:
             logging.exception('list_datasets , ' + str(e))
+            raise Error(e)
+
+
+
+    def list_schedules_notebook(self, n=10, rule_prefix = None):
+        """Return a pandas data frame of the schedule rules.
+
+        Args:
+            n (int): The number of rules to return or all rules if 0 (default: 0)
+            rule_prefix (str): If not None, return only rules whose names begin with the prefix (default: None)
+        """
+
+        try:
+
+            params = {}
+            if n:
+                params['n'] = n
+            if rule_prefix:
+                params['rule_prefix'] = rule_prefix
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/aws/listSchedulesNotebook'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook schedules...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    rJson = json.loads(r.text)
+                    logging.info('list notebook schedules finished successfully.')
+                    try:
+                        df = pd.read_json(rJson)
+                        return df
+                    except Exception as e:
+                        logging.info('No notebook schedules found.')
+                else:
+                    logging.exception('Could not find proper output, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('list_schedules_notebook , ' + str(e))
+            raise Error(e)
+
+
+
+    def describe_notebook_run(self, job_name= None):
+        """Stop the named processing job.
+        Args:
+        job_name (string): The name of the job to stop. use list_runs_notebook to get specific notebook Job name
+        Returns:
+          A dictionary with keys for each element of the job description. For example::
+
+          {'Notebook': 'test.ipynb',
+           'Rule': '',
+           'Parameters': '{"input": "s3://notebook-testing/const.txt"}',
+           'Job': 'papermill-test-2020-10-21-20-00-11',
+           'Status': 'Completed',
+           'Failure': None,
+           'Created': datetime.datetime(2020, 10, 21, 13, 0, 12, 817000, tzinfo=tzlocal()),
+           'Start': datetime.datetime(2020, 10, 21, 13, 4, 1, 58000, tzinfo=tzlocal()),
+           'End': datetime.datetime(2020, 10, 21, 13, 4, 55, 710000, tzinfo=tzlocal()),
+           'Elapsed': datetime.timedelta(seconds=54, microseconds=652000),
+           'Result': 's3://dataplate/output/test-2020-10-21-20-00-11.ipynb',
+           'Input': 's3://dataplate/input/notebook-2020-10-21-20-00-08.ipynb',
+           'Image': 'notebook-runner',
+           'Instance': 'ml.m5.large',
+           'Role': 'BasicExecuteNotebookRole-us-west-2'}
+        """
+
+        try:
+
+            params = {}
+            if job_name:
+                params['jobname'] = job_name
+            else:
+                raise Error('Not a valid job name, use list_runs_notebook function to get specific notebook Job name.')
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/aws/describeNotebookRun'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook details...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    try:
+                        rJson = json.loads(r.text)
+                        # res = ast.literal_eval(r.text)
+                        logging.info('describe notebook run finished successfully.')
+                        return rJson
+                    except Exception as e:
+                        logging.info('Invalid notebook details found.')
+                else:
+                    logging.exception('Could not find proper result, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('describe_notebook_run , ' + str(e))
+            raise Error(e)
+
+
+    def wait_for_complete(self, job_name, progress=True, sleep_time=10):
+        """Wait for a notebook execution job to complete.
+
+        Args:
+          job_name (str):
+            The name of the SageMaker Processing Job executing the notebook. (Required)
+          progress (boolean):
+            If True, print a period after every poll attempt. (Default: True)
+          sleep_time (int):
+            The number of seconds between polls. (Default: 10)
+
+        Returns:
+          A tuple with the job status and the failure message if any.
+        """
+
+        done = False
+        while not done:
+            if progress:
+                print(".", end="")
+            desc = self.describe_notebook_run(job_name=job_name)
+            status = desc["Status"]
+            if status != "InProgress":
+                done = True
+            else:
+                time.sleep(sleep_time)
+        if progress:
+            print()
+        return status, desc.get("FailureReason")
+
+
+    def stop_schedule_notebook(self, rule_name= None):
+        """Delete an existing notebook schedule rule.
+        Args:
+            rule_name (str): The name of the schedule rule (required).
+        """
+
+        try:
+
+            params = {}
+            if rule_name:
+                params['rule_name'] = rule_name
+            else:
+                raise Error('Not a valid rule name, use list_schedules_notebook function to get specific notebook schedule rule name.')
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('sending request...')
+                r = self.session.post( \
+                    '{}/api/aws/stopSchedule'.format(self.base_url), params=params, data="",
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('analysing notebook schedule stops...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    logging.info(r.text)
+                    return
+                else:
+                    logging.exception('Could not find proper result, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('stop_schedule_notebook , ' + str(e))
+            raise Error(e)
+
+
+
+    def schedule_notebook(self, notebook_file_path, rule_name = None, schedule = None, instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= []):
+        """Create a schedule for running a notebook in SageMaker Processing.
+
+            Creates a CloudWatch Event rule to invoke the installed Lambda either on the provided schedule or in response
+            to the provided event. \
+
+            :meth:`schedule` can upload a local notebook file to run or use one previously uploaded to S3. 
+            To find jobs run by the schedule, see :meth:`list_runs_notebook` using the `rule` argument to filter to 
+            a specific rule. To download the results, see :meth:`download_notebook_result` 
+
+            job = run.schedule_notebook(notebook="powers.ipynb", rule_name="Powers", schedule="rate(1 hour)")
+
+            Args:
+                notebook (str): The notebook file path.
+                rule_name (str): The name of the rule for CloudWatch Events (required).
+                schedule (str): A schedule string which defines when the job should be run. For details, 
+                                see https://docs.aws.amazon.com/AmazonCloudWatch/latest/events/ScheduledEvents.html 
+                                (default: None)
+                input_path (str): The S3 object containing the notebook. If this is None, the `notebook` argument is
+                                  taken as a local file to upload (default: None).
+                parameters (dict): The dictionary of parameters to pass to the notebook (default: {}).
+                instance_type (str): The SageMaker instance to use for executing the job (default: ml.m5.large).
+                max_time_limit_minutes : maximum minutes to run before force stop
+                securityGroupIds : a list of securityGroup Ids of aws for the processing job to communicate with, in case communication with other resources, e.g. internal dataplate service, is needed
+                subnets : a list of subnets of aws for the processing job to communicate with
+            """
+        try:
+            if not notebook_file_path or not os.path.isfile(notebook_file_path) or not notebook_file_path.endswith('.ipynb'):
+                raise FileNotFoundError(f'notebook file is not legal/valid : {notebook_file_path if notebook_file_path else "None"}')
+
+            f = open(notebook_file_path, 'r')
+            if f:
+                # Reading from file
+                notebook_json_data = json.loads(f.read())
+                # Closing file
+                f.close()
+
+            if not notebook_json_data or len(json.dumps(notebook_json_data)) < 10 or not 'cells' in notebook_json_data:
+                raise Error(f'notebook file is not legal : {notebook_file_path if notebook_file_path else "None"}')
+
+            notebook_name = os.path.basename(notebook_file_path)
+            params = {}
+            if instance_type:
+                params['instance_type'] = instance_type
+            if parameters:
+                params['parameters'] = parameters
+            if max_time_limit_minutes:
+                params['timelimit_minutes'] = max_time_limit_minutes
+            params['SecurityGroupIds'] = securityGroupIds
+            params['Subnets'] = subnets
+            params['notebook_name'] = notebook_name if notebook_name else ""
+            if not rule_name:
+                raise Error(f'rule_name is required')
+            if not schedule:
+                raise Error(f'schedule is required, see https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-create-rule-schedule.html#eb-cron-expressions')
+            params['rule_name'] = rule_name
+            params['schedule'] = schedule
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('Uploading data...')
+                r = self.session.post( \
+                    '{}/api/aws/scheduleNotebook'.format(self.base_url), params=params, data=json.dumps(notebook_json_data),
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('scheduling notebook, processing...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    rJson = json.loads(r.text)
+                    logging.info('schedule notebook finished successfully.')
+                    return json.dumps(rJson)
+                else:
+                    logging.exception('Could not find proper output, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('schedule_notebook , ' + str(e))
             raise Error(e)
\ No newline at end of file
